package dmLab.utils.matrix;

/**
 * <p>Title: DM System </p>
 * <p>Description: Data Mining syetem for experiments.</p>
 * <p>Copyright: Copyright (c) 2002 IPI PAN & Mike242 Soft</p>
 * <p>Company: IPI PAN & Mike242 Soft</p>
 * @author Michal Draminski
 * @version 1.0
 */


public class Measure
{
    public static int ACC=1;
    public static int WACC=2;

//  ***********************************************
//  *** calculates weighted accuracy from input confusion matrix
    public static double calcWAcc(double confusionMatrix[][])
    {
        int rowSum,rows=0;
        double wAcc=0;
        for (int i=0;i<confusionMatrix[0].length;i++)
        {
            rowSum=0;
            for (int j=0;j<confusionMatrix[0].length;j++)
                rowSum+=confusionMatrix[i][j];
            if(rowSum!=0)
            {
                wAcc+=((double)confusionMatrix[i][i]/(double)rowSum );
                rows++;
            }
        }
        wAcc=wAcc/(double)(rows);
        return wAcc;
    }
//  ***********************************************
//  *** clculates regular accuracy from input confusion matrix
    public static double calcAcc(double confusionMatrix[][])
    {
        int sum=0;
        int trueDecision=0;

        for (int i=0;i<confusionMatrix[0].length;i++)
        {
            for (int j=0;j<confusionMatrix.length;j++)
            {
                sum+=confusionMatrix[i][j];
                if(i==j)
                    trueDecision+=confusionMatrix[i][j];
            }
        }
        return (double)trueDecision/(double)sum;
    }
//  ***********************************************
//  *** calculates true positive rate from input confusion matrix
    public static double calcTPRate(double confusionMatrix[][],int valueIndex)
    {
        int rowSum=0;
        for (int j=0;j<confusionMatrix[0].length;j++)
            rowSum+=confusionMatrix[valueIndex][j];

        return (double)confusionMatrix[valueIndex][valueIndex]/(double)rowSum;
    }
//  ***********************************************
//  **** calculates false positive rate from input confusion matrix
    public static double calcFPRate(double confusionMatrix[][],int valueIndex)
    {
        int rowSum=0;
        int FP=0;
        for (int i=0;i<confusionMatrix[0].length;i++)
        {
            if(i!=valueIndex)
            {
                FP+=confusionMatrix[i][valueIndex];
                for (int j=0;j<confusionMatrix.length;j++)
                    rowSum+=confusionMatrix[i][j];
            }
        }
        return (double)FP/(double)rowSum;
    }
//  ***********************************************
//  *** calculates numebr of events that were misclassified
    public static int errors(double confusionMatrix[][])
    {
        int errors=0;

        for (int i=0;i<confusionMatrix[0].length;i++)
        {
            for (int j=0;j<confusionMatrix.length;j++)
                if(i!=j)
                    errors+=confusionMatrix[i][j];
        }
        return errors;
    }
//  *********************************************** 
    public static String toString(double confusionMatrix[][])
    {
        StringBuffer tmp=new StringBuffer();
        for (int i=0;i<confusionMatrix[0].length;i++)
        {
            for (int j=0;j<confusionMatrix.length;j++)
            {
                tmp.append((float)confusionMatrix[i][j]+"\t");
            }
            tmp.append("\n");
        }
        return tmp.toString();
        
    }
//  ***********************************************    
}
